@@ -1,7 +1,7 @@
 /*
  * cmd_defs.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -49,4 +49,7 @@ gboolean cmd_valid_tag(const char *const str);
 
 void command_docgen(void);
 
+GList* cmd_search_index_all(char *term);
+GList* cmd_search_index_any(char *term);
+
 #endif
