@@ -1,7 +1,7 @@
 /*
  * common.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -46,6 +46,7 @@
 #include <curl/curl.h>
 #include <curl/easy.h>
 #include <glib.h>
+#include <gio/gio.h>
 
 #ifdef HAVE_NCURSESW_NCURSES_H
 #include <ncursesw/ncurses.h>
@@ -55,7 +56,6 @@
 
 #include "log.h"
 #include "common.h"
-#include "tools/p_sha1.h"
 
 struct curl_data_t
 {
@@ -63,58 +63,8 @@ struct curl_data_t
     size_t size;
 };
 
-static unsigned long unique_id = 0;
-
 static size_t _data_callback(void *ptr, size_t size, size_t nmemb, void *data);
 
-// taken from glib 2.30.3
-gchar*
-p_utf8_substring(const gchar *str, glong start_pos, glong end_pos)
-{
-    gchar *start, *end, *out;
-
-    start = g_utf8_offset_to_pointer (str, start_pos);
-    end = g_utf8_offset_to_pointer (start, end_pos - start_pos);
-
-    out = g_malloc (end - start + 1);
-    memcpy (out, start, end - start);
-    out[end - start] = 0;
-
-    return out;
-}
-
-void
-p_slist_free_full(GSList *items, GDestroyNotify free_func)
-{
-    g_slist_foreach (items, (GFunc) free_func, NULL);
-    g_slist_free (items);
-}
-
-void
-p_list_free_full(GList *items, GDestroyNotify free_func)
-{
-    g_list_foreach (items, (GFunc) free_func, NULL);
-    g_list_free (items);
-}
-
-gboolean
-p_hash_table_add(GHashTable *hash_table, gpointer key)
-{
-    // doesn't handle when key exists, but value == NULL
-    gpointer found = g_hash_table_lookup(hash_table, key);
-    g_hash_table_replace(hash_table, key, key);
-
-    return (found == NULL);
-}
-
-gboolean
-p_hash_table_contains(GHashTable *hash_table, gconstpointer key)
-{
-    // doesn't handle when key exists, but value == NULL
-    gpointer found = g_hash_table_lookup(hash_table, key);
-    return (found != NULL);
-}
-
 gboolean
 create_dir(char *name)
 {
@@ -155,28 +105,16 @@ mkdir_recursive(const char *dir)
 }
 
 gboolean
-copy_file(const char *const sourcepath, const char *const targetpath)
+copy_file(const char *const sourcepath, const char *const targetpath, const gboolean overwrite_existing)
 {
-    int ch;
-    FILE *source = fopen(sourcepath, "rb");
-    if (source == NULL) {
-        return FALSE;
-    }
-
-    FILE *target = fopen(targetpath, "wb");
-    if (target == NULL) {
-        fclose(source);
-        return FALSE;
-    }
-
-    while((ch = fgetc(source)) != EOF) {
-        fputc(ch, target);
-    }
-
-    fclose(source);
-    fclose(target);
-
-    return TRUE;
+    GFile *source = g_file_new_for_path(sourcepath);
+    GFile *dest = g_file_new_for_path(targetpath);
+    GError *error = NULL;
+    GFileCopyFlags flags = overwrite_existing ? G_FILE_COPY_OVERWRITE : G_FILE_COPY_NONE;
+    gboolean success = g_file_copy (source, dest, flags, NULL, NULL, NULL, &error);
+    g_object_unref(source);
+    g_object_unref(dest);
+    return success;
 }
 
 char*
@@ -380,46 +318,6 @@ release_is_new(char *found_version)
     }
 }
 
-char*
-create_unique_id(char *prefix)
-{
-    char *result = NULL;
-    GString *result_str = g_string_new("");
-
-    unique_id++;
-    if (prefix) {
-        g_string_printf(result_str, "prof_%s_%lu", prefix, unique_id);
-    } else {
-        g_string_printf(result_str, "prof_%lu", unique_id);
-    }
-    result = result_str->str;
-    g_string_free(result_str, FALSE);
-
-    return result;
-}
-
-void
-reset_unique_id(void)
-{
-    unique_id = 0;
-}
-
-char*
-p_sha1_hash(char *str)
-{
-    P_SHA1_CTX ctx;
-    uint8_t digest[20];
-    uint8_t *input = (uint8_t*)malloc(strlen(str) + 1);
-    memcpy(input, str, strlen(str) + 1);
-
-    P_SHA1_Init(&ctx);
-    P_SHA1_Update(&ctx, input, strlen(str));
-    P_SHA1_Final(&ctx, digest);
-
-    free(input);
-    return g_base64_encode(digest, sizeof(digest));
-}
-
 static size_t
 _data_callback(void *ptr, size_t size, size_t nmemb, void *data)
 {
@@ -515,11 +413,18 @@ prof_occurrences(const char *const needle, const char *const haystack, int offse
             gchar *needle_last_ch = g_utf8_offset_to_pointer(needle, g_utf8_strlen(needle, -1)- 1);
             int needle_last_ch_len = mblen(needle_last_ch, MB_CUR_MAX);
 
-            gchar *haystack_before_ch = g_utf8_prev_char(haystack_curr);
-            gchar *haystack_after_ch = g_utf8_next_char(haystack_curr + strlen(needle) - needle_last_ch_len);
+            gunichar before = 0;
+            gchar *haystack_before_ch = g_utf8_find_prev_char(haystack, haystack_curr);
+            if (haystack_before_ch) {
+                before = g_utf8_get_char(haystack_before_ch);
+            }
+
+            gunichar after = 0;
+            gchar *haystack_after_ch = g_utf8_find_next_char(haystack_curr + strlen(needle) - needle_last_ch_len, NULL);
+            if (haystack_after_ch) {
+                after = g_utf8_get_char(haystack_after_ch);
+            }
 
-            gunichar before = g_utf8_get_char(haystack_before_ch);
-            gunichar after = g_utf8_get_char(haystack_after_ch);
             if (!g_unichar_isalnum(before) && !g_unichar_isalnum(after)) {
                 *result = g_slist_append(*result, GINT_TO_POINTER(offset));
             }
@@ -536,3 +441,45 @@ prof_occurrences(const char *const needle, const char *const haystack, int offse
     return *result;
 }
 
+int
+is_regular_file(const char *path)
+{
+    struct stat st;
+    stat(path, &st);
+    return S_ISREG(st.st_mode);
+}
+
+int
+is_dir(const char *path)
+{
+    struct stat st;
+    stat(path, &st);
+    return S_ISDIR(st.st_mode);
+}
+
+void
+get_file_paths_recursive(const char *path, GSList **contents)
+{
+    if (!is_dir(path)) {
+        return;
+    }
+
+    GDir* directory = g_dir_open(path, 0, NULL);
+    const gchar *entry = g_dir_read_name(directory);
+    while (entry) {
+        GString *full = g_string_new(path);
+        if (!g_str_has_suffix(full->str, "/")) {
+            g_string_append(full, "/");
+        }
+        g_string_append(full, entry);
+
+        if (is_dir(full->str)) {
+            get_file_paths_recursive(full->str, contents);
+        } else if (is_regular_file(full->str)) {
+            *contents = g_slist_append(*contents, full->str);
+        }
+
+        g_string_free(full, FALSE);
+        entry = g_dir_read_name(directory);
+    }
+}
