@@ -1,7 +1,7 @@
 /*
  * common.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -40,20 +40,6 @@
 
 #include <glib.h>
 
-#if !GLIB_CHECK_VERSION(2,28,0)
-#define g_slist_free_full(items, free_func)         p_slist_free_full(items, free_func)
-#define g_list_free_full(items, free_func)          p_list_free_full(items, free_func)
-#endif
-
-#if !GLIB_CHECK_VERSION(2,30,0)
-#define g_utf8_substring(str, start_pos, end_pos)   p_utf8_substring(str, start_pos, end_pos)
-#endif
-
-#if !GLIB_CHECK_VERSION(2,32,0)
-#define g_hash_table_add(hash_table, key)           p_hash_table_add(hash_table, key)
-#define g_hash_table_contains(hash_table, key)      p_hash_table_contains(hash_table, key)
-#endif
-
 #ifndef NOTIFY_CHECK_VERSION
 #define notify_notification_new(summary, body, icon) notify_notification_new(summary, body, icon, NULL)
 #endif
@@ -94,15 +80,9 @@ typedef enum {
     RESOURCE_XA
 } resource_presence_t;
 
-gchar* p_utf8_substring(const gchar *str, glong start_pos, glong end_pos);
-void p_slist_free_full(GSList *items, GDestroyNotify free_func);
-void p_list_free_full(GList *items, GDestroyNotify free_func);
-gboolean p_hash_table_add(GHashTable *hash_table, gpointer key);
-gboolean p_hash_table_contains(GHashTable *hash_table, gconstpointer key);
-
 gboolean create_dir(char *name);
 gboolean mkdir_recursive(const char *dir);
-gboolean copy_file(const char *const src, const char *const target);
+gboolean copy_file(const char *const src, const char *const target, const gboolean overwrite_existing);
 char* str_replace(const char *string, const char *substr, const char *replacement);
 int str_contains(const char str[], int size, char ch);
 gboolean strtoi_range(char *str, int *saveptr, int min, int max, char **err_msg);
@@ -112,10 +92,6 @@ char* file_getline(FILE *stream);
 char* release_get_latest(void);
 gboolean release_is_new(char *found_version);
 
-char* p_sha1_hash(char *str);
-char* create_unique_id(char *prefix);
-void reset_unique_id(void);
-
 char* get_file_or_linked(char *loc, char *basedir);
 char* strip_arg_quotes(const char *const input);
 gboolean is_notify_enabled(void);
@@ -123,4 +99,8 @@ gboolean is_notify_enabled(void);
 GSList* prof_occurrences(const char *const needle, const char *const haystack, int offset, gboolean whole_word,
     GSList **result);
 
+int is_regular_file(const char *path);
+int is_dir(const char *path);
+void get_file_paths_recursive(const char *directory, GSList **contents);
+
 #endif
