@@ -1,7 +1,7 @@
 /*
  * accounts.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -96,15 +96,15 @@ accounts_close(void)
 }
 
 char*
-accounts_find_enabled(const char *const prefix)
+accounts_find_enabled(const char *const prefix, gboolean previous)
 {
-    return autocomplete_complete(enabled_ac, prefix, TRUE);
+    return autocomplete_complete(enabled_ac, prefix, TRUE, previous);
 }
 
 char*
-accounts_find_all(const char *const prefix)
+accounts_find_all(const char *const prefix, gboolean previous)
 {
-    return autocomplete_complete(all_ac, prefix, TRUE);
+    return autocomplete_complete(all_ac, prefix, TRUE, previous);
 }
 
 void
@@ -291,7 +291,9 @@ accounts_get_account(const char *const name)
         gchar *tls_policy = g_key_file_get_string(accounts, name, "tls.policy", NULL);
         if (tls_policy && ((g_strcmp0(tls_policy, "force") != 0) &&
                 (g_strcmp0(tls_policy, "allow") != 0) &&
-                (g_strcmp0(tls_policy, "disable") != 0))) {
+                (g_strcmp0(tls_policy, "trust") != 0) &&
+                (g_strcmp0(tls_policy, "disable") != 0) &&
+                (g_strcmp0(tls_policy, "legacy") != 0))) {
             g_free(tls_policy);
             tls_policy = NULL;
         }
