@@ -1,7 +1,7 @@
 /*
  * preferences.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -49,8 +49,8 @@ typedef enum {
     PREF_BEEP,
     PREF_VERCHECK,
     PREF_THEME,
-    PREF_TITLEBAR_SHOW,
-    PREF_TITLEBAR_GOODBYE,
+    PREF_WINTITLE_SHOW,
+    PREF_WINTITLE_GOODBYE,
     PREF_FLASH,
     PREF_TRAY,
     PREF_TRAY_READ,
@@ -84,15 +84,15 @@ typedef enum {
     PREF_ROSTER_ROOMS_BY,
     PREF_ROSTER_ROOMS_ORDER,
     PREF_ROSTER_ROOMS_UNREAD,
+    PREF_ROSTER_ROOMS_SERVER,
     PREF_ROSTER_PRIVATE,
     PREF_MUC_PRIVILEGES,
     PREF_PRESENCE,
     PREF_WRAP,
-    PREF_WINS_AUTO_TIDY,
     PREF_TIME_CONSOLE,
     PREF_TIME_CHAT,
     PREF_TIME_MUC,
-    PREF_TIME_MUCCONFIG,
+    PREF_TIME_CONFIG,
     PREF_TIME_PRIVATE,
     PREF_TIME_XMLCONSOLE,
     PREF_TIME_STATUSBAR,
@@ -141,6 +141,13 @@ typedef enum {
     PREF_CONSOLE_PRIVATE,
     PREF_CONSOLE_CHAT,
     PREF_BOOKMARK_INVITE,
+    PREF_PLUGINS_SOURCEPATH,
+    PREF_ROOM_LIST_CACHE,
+    PREF_STATUSBAR_SHOW_NAME,
+    PREF_STATUSBAR_SHOW_NUMBER,
+    PREF_STATUSBAR_SELF,
+    PREF_STATUSBAR_CHAT,
+    PREF_STATUSBAR_ROOM,
 } preference_t;
 
 typedef struct prof_alias_t {
@@ -148,16 +155,23 @@ typedef struct prof_alias_t {
     gchar *value;
 } ProfAlias;
 
+typedef struct prof_winplacement_t {
+    int titlebar_pos;
+    int mainwin_pos;
+    int statusbar_pos;
+    int inputwin_pos;
+} ProfWinPlacement;
+
 void prefs_load(void);
 void prefs_close(void);
 
 char* prefs_find_login(char *prefix);
 void prefs_reset_login_search(void);
 
-char* prefs_autocomplete_boolean_choice(const char *const prefix);
+char* prefs_autocomplete_boolean_choice(const char *const prefix, gboolean previous);
 void prefs_reset_boolean_choice(void);
 
-char* prefs_autocomplete_room_trigger(const char *const prefix);
+char* prefs_autocomplete_room_trigger(const char *const prefix, gboolean previous);
 void prefs_reset_room_trigger_ac(void);
 
 gint prefs_get_gone(void);
@@ -178,6 +192,11 @@ gint prefs_get_autoping_timeout(void);
 gint prefs_get_inpblock(void);
 void prefs_set_inpblock(gint value);
 
+void prefs_set_statusbartabs(gint value);
+gint prefs_get_statusbartabs(void);
+void prefs_set_statusbartablen(gint value);
+gint prefs_get_statusbartablen(void);
+
 void prefs_set_occupants_size(gint value);
 gint prefs_get_occupants_size(void);
 void prefs_set_roster_size(gint value);
@@ -239,6 +258,20 @@ gboolean prefs_add_room_notify_trigger(const char * const text);
 gboolean prefs_remove_room_notify_trigger(const char * const text);
 GList* prefs_get_room_notify_triggers(void);
 
+ProfWinPlacement* prefs_get_win_placement(void);
+void prefs_free_win_placement(ProfWinPlacement *placement);
+
+gboolean prefs_titlebar_pos_up(void);
+gboolean prefs_titlebar_pos_down(void);
+gboolean prefs_mainwin_pos_up(void);
+gboolean prefs_mainwin_pos_down(void);
+gboolean prefs_statusbar_pos_up(void);
+gboolean prefs_statusbar_pos_down(void);
+gboolean prefs_inputwin_pos_up(void);
+gboolean prefs_inputwin_pos_down(void);
+ProfWinPlacement* prefs_create_profwin_placement(int titlebar, int mainwin, int statusbar, int inputwin);
+void prefs_save_win_placement(ProfWinPlacement *placement);
+
 gboolean prefs_get_boolean(preference_t pref);
 void prefs_set_boolean(preference_t pref, gboolean value);
 char* prefs_get_string(preference_t pref);
