@@ -1,7 +1,7 @@
 /*
  * api.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -85,7 +85,7 @@ api_cons_show_themed(const char *const group, const char *const key, const char
     char *parsed = str_replace(message, "\r\n", "\n");
     theme_item_t themeitem = plugin_themes_get(group, key, def);
     ProfWin *console = wins_get_console();
-    win_print(console, '-', 0, NULL, 0, themeitem, "", parsed);
+    win_println(console, themeitem, '-', "%s", parsed);
 
     free(parsed);
 
@@ -333,7 +333,7 @@ api_win_create(
     // set status bar active
     ProfPluginWin *pluginwin = wins_get_plugin(tag);
     int num = wins_get_num((ProfWin*)pluginwin);
-    status_bar_active(num);
+    status_bar_active(num, WIN_PLUGIN, pluginwin->tag);
 }
 
 int
@@ -374,7 +374,7 @@ api_win_show(const char *tag, const char *line)
     }
 
     ProfWin *window = (ProfWin*)pluginwin;
-    win_print(window, '!', 0, NULL, 0, 0, "", line);
+    win_println(window, THEME_DEFAULT, '!', "%s", line);
 
     return 1;
 }
@@ -399,7 +399,7 @@ api_win_show_themed(const char *tag, const char *const group, const char *const
 
     theme_item_t themeitem = plugin_themes_get(group, key, def);
     ProfWin *window = (ProfWin*)pluginwin;
-    win_print(window, '!', 0, NULL, 0, themeitem, "", line);
+    win_println(window, themeitem, '!', "%s", line);
 
     return 1;
 }
@@ -515,7 +515,7 @@ api_encryption_reset(const char *const barejid)
 #ifdef HAVE_LIBGPGME
     if (chatwin->pgp_send) {
         chatwin->pgp_send = FALSE;
-        ui_current_print_formatted_line('!', 0, "PGP encryption disabled.");
+        win_println((ProfWin*)chatwin, THEME_DEFAULT, '!', "PGP encryption disabled.");
     }
 #endif
 
@@ -777,7 +777,7 @@ api_chat_show(const char *const barejid, const char *message)
     }
 
     char *parsed = str_replace(message, "\r\n", "\n");
-    win_println((ProfWin*)chatwin, 0, parsed);
+    win_println((ProfWin*)chatwin, THEME_TEXT, '-', "%s", parsed);
     free(parsed);
 
     return 1;
@@ -816,7 +816,7 @@ api_chat_show_themed(const char *const barejid, const char *const group, const c
     char *parsed = str_replace(message, "\r\n", "\n");
     theme_item_t themeitem = plugin_themes_get(group, key, def);
 
-    win_print((ProfWin*)chatwin, show_ch, 0, NULL, 0, themeitem, "", parsed);
+    win_println((ProfWin*)chatwin, themeitem, show_ch, "%s", parsed);
     free(parsed);
 
     return 1;
@@ -842,7 +842,7 @@ api_room_show(const char *const roomjid, const char *message)
     }
 
     char *parsed = str_replace(message, "\r\n", "\n");
-    win_println((ProfWin*)mucwin, 0, parsed);
+    win_println((ProfWin*)mucwin, THEME_TEXT, '-', "%s", parsed);
     free(parsed);
 
     return 1;
@@ -881,7 +881,7 @@ api_room_show_themed(const char *const roomjid, const char *const group, const c
     char *parsed = str_replace(message, "\r\n", "\n");
     theme_item_t themeitem = plugin_themes_get(group, key, def);
 
-    win_print((ProfWin*)mucwin, show_ch, 0, NULL, 0, themeitem, "", parsed);
+    win_println((ProfWin*)mucwin, themeitem, show_ch, "%s", parsed);
     free(parsed);
 
     return 1;
