@@ -1,7 +1,7 @@
 /*
  * plugins.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -42,6 +42,11 @@ typedef enum {
     LANG_C
 } lang_t;
 
+typedef struct prof_plugins_install_t {
+    GSList *installed;
+    GSList *failed;
+} PluginsInstallResult;
+
 typedef struct prof_plugin_t {
     char *name;
     lang_t lang;
@@ -103,13 +108,20 @@ typedef struct prof_plugin_t {
 void plugins_init(void);
 GSList *plugins_unloaded_list(void);
 GList *plugins_loaded_list(void);
-char* plugins_autocomplete(const char *const input);
+char* plugins_autocomplete(const char *const input, gboolean previous);
 void plugins_reset_autocomplete(void);
 void plugins_shutdown(void);
 
-gboolean plugins_install(const char *const plugin_name, const char *const filename);
+void plugins_free_install_result(PluginsInstallResult *result);
+
+gboolean plugins_install(const char *const plugin_name, const char *const filename, GString * error_message);
+gboolean plugins_uninstall(const char *const plugin_name);
+gboolean plugins_update(const char *const plugin_name, const char *const filename, GString * error_message);
+PluginsInstallResult* plugins_install_all(const char *const path);
 gboolean plugins_load(const char *const name);
+GSList* plugins_load_all(void);
 gboolean plugins_unload(const char *const name);
+gboolean plugins_unload_all(void);
 gboolean plugins_reload(const char *const name);
 void plugins_reload_all(void);
 
