@@ -1,7 +1,7 @@
 /*
  * mucwin.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -51,14 +51,14 @@ mucwin_role_change(ProfMucWin *mucwin, const char *const role, const char *const
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "Your role has been changed to: %s", role);
+    win_print(window, THEME_ROOMINFO, '!', "Your role has been changed to: %s", role);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -68,14 +68,14 @@ mucwin_affiliation_change(ProfMucWin *mucwin, const char *const affiliation, con
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "Your affiliation has been changed to: %s", affiliation);
+    win_print(window, THEME_ROOMINFO, '!', "Your affiliation has been changed to: %s", affiliation);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -85,14 +85,14 @@ mucwin_role_and_affiliation_change(ProfMucWin *mucwin, const char *const role, c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "Your role and affiliation have been changed, role: %s, affiliation: %s", role, affiliation);
+    win_print(window, THEME_ROOMINFO, '!', "Your role and affiliation have been changed, role: %s, affiliation: %s", role, affiliation);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 
@@ -103,14 +103,14 @@ mucwin_occupant_role_change(ProfMucWin *mucwin, const char *const nick, const ch
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "%s's role has been changed to: %s", nick, role);
+    win_print(window, THEME_ROOMINFO, '!', "%s's role has been changed to: %s", nick, role);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -120,14 +120,14 @@ mucwin_occupant_affiliation_change(ProfMucWin *mucwin, const char *const nick, c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "%s's affiliation has been changed to: %s", nick, affiliation);
+    win_print(window, THEME_ROOMINFO, '!', "%s's affiliation has been changed to: %s", nick, affiliation);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -137,14 +137,14 @@ mucwin_occupant_role_and_affiliation_change(ProfMucWin *mucwin, const char *cons
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "%s's role and affiliation have been changed, role: %s, affiliation: %s", nick, role, affiliation);
+    win_print(window, THEME_ROOMINFO, '!', "%s's role and affiliation have been changed, role: %s, affiliation: %s", nick, role, affiliation);
     if (actor) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", by: %s", actor);
+        win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
     if (reason) {
-        win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ROOMINFO, "", ", reason: %s", reason);
+        win_append(window, THEME_ROOMINFO, ", reason: %s", reason);
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -153,8 +153,8 @@ mucwin_room_info_error(ProfMucWin *mucwin, const char *const error)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, 0, "", "Room info request failed: %s", error);
-    win_print(window, '-', 0, NULL, 0, 0, "", "");
+    win_println(window, THEME_DEFAULT, '!', "Room info request failed: %s", error);
+    win_println(window, THEME_DEFAULT, '-', "");
 }
 
 void
@@ -166,7 +166,7 @@ mucwin_room_disco_info(ProfMucWin *mucwin, GSList *identities, GSList *features)
     if ((identities && (g_slist_length(identities) > 0)) ||
         (features && (g_slist_length(features) > 0))) {
         if (identities) {
-            win_print(window, '!', 0, NULL, 0, 0, "", "Identities:");
+            win_println(window, THEME_DEFAULT, '!', "Identities:");
         }
         while (identities) {
             DiscoIdentity *identity = identities->data;  // anme trpe, cat
@@ -182,19 +182,19 @@ mucwin_room_disco_info(ProfMucWin *mucwin, GSList *identities, GSList *features)
             if (identity->category) {
                 identity_str = g_string_append(identity_str, identity->category);
             }
-            win_print(window, '!', 0, NULL, 0, 0, "", identity_str->str);
+            win_println(window, THEME_DEFAULT, '!', "%s", identity_str->str);
             g_string_free(identity_str, TRUE);
             identities = g_slist_next(identities);
         }
 
         if (features) {
-            win_print(window, '!', 0, NULL, 0, 0, "", "Features:");
+            win_println(window, THEME_DEFAULT, '!', "Features:");
         }
         while (features) {
-            win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", features->data);
+            win_println(window, THEME_DEFAULT, '!', "  %s", features->data);
             features = g_slist_next(features);
         }
-        win_print(window, '-', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '-', "");
     }
 }
 
@@ -206,16 +206,16 @@ mucwin_roster(ProfMucWin *mucwin, GList *roster, const char *const presence)
     ProfWin *window = (ProfWin*)mucwin;
     if ((roster == NULL) || (g_list_length(roster) == 0)) {
         if (presence == NULL) {
-            win_print(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Room is empty.");
+            win_println(window, THEME_ROOMINFO, '!', "Room is empty.");
         } else {
-            win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "No occupants %s.", presence);
+            win_println(window, THEME_ROOMINFO, '!', "No occupants %s.", presence);
         }
     } else {
         int length = g_list_length(roster);
         if (presence == NULL) {
-            win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "%d occupants: ", length);
+            win_print(window, THEME_ROOMINFO, '!', "%d occupants: ", length);
         } else {
-            win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "%d %s: ", length, presence);
+            win_print(window, THEME_ROOMINFO, '!', "%d %s: ", length, presence);
         }
 
         while (roster) {
@@ -223,16 +223,15 @@ mucwin_roster(ProfMucWin *mucwin, GList *roster, const char *const presence)
             const char *presence_str = string_from_resource_presence(occupant->presence);
 
             theme_item_t presence_colour = theme_main_presence_attrs(presence_str);
-            win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, presence_colour, "", "%s", occupant->nick);
+            win_append(window, presence_colour, "%s", occupant->nick);
 
             if (roster->next) {
-                win_print(window, '!', 0, NULL, NO_DATE | NO_EOL, 0, "", ", ");
+                win_append(window, THEME_DEFAULT, ", ");
             }
 
             roster = g_list_next(roster);
         }
-        win_print(window, '!', 0, NULL, NO_DATE, THEME_ONLINE, "", "");
-
+        win_appendln(window, THEME_ONLINE, "");
     }
 }
 
@@ -242,7 +241,7 @@ mucwin_occupant_offline(ProfMucWin *mucwin, const char *const nick)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_OFFLINE, "", "<- %s has left the room.", nick);
+    win_println(window, THEME_OFFLINE, '!', "<- %s has left the room.", nick);
 }
 
 void
@@ -263,7 +262,7 @@ mucwin_occupant_kicked(ProfMucWin *mucwin, const char *const nick, const char *c
         g_string_append(message, reason);
     }
 
-    win_vprint(window, '!', 0, NULL, 0, THEME_OFFLINE, "", "<- %s", message->str);
+    win_println(window, THEME_OFFLINE, '!', "<- %s", message->str);
     g_string_free(message, TRUE);
 }
 
@@ -285,7 +284,7 @@ mucwin_occupant_banned(ProfMucWin *mucwin, const char *const nick, const char *c
         g_string_append(message, reason);
     }
 
-    win_vprint(window, '!', 0, NULL, 0, THEME_OFFLINE, "", "<- %s", message->str);
+    win_println(window, THEME_OFFLINE, '!', "<- %s", message->str);
     g_string_free(message, TRUE);
 }
 
@@ -296,16 +295,16 @@ mucwin_occupant_online(ProfMucWin *mucwin, const char *const nick, const char *c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ONLINE, "", "-> %s has joined the room", nick);
+    win_print(window, THEME_ONLINE, '!', "-> %s has joined the room", nick);
     if (prefs_get_boolean(PREF_MUC_PRIVILEGES)) {
         if (role) {
-            win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ONLINE, "", ", role: %s", role);
+            win_append(window, THEME_ONLINE, ", role: %s", role);
         }
         if (affiliation) {
-            win_vprint(window, '!', 0, NULL, NO_DATE | NO_EOL, THEME_ONLINE, "", ", affiliation: %s", affiliation);
+            win_append(window, THEME_ONLINE, ", affiliation: %s", affiliation);
         }
     }
-    win_print(window, '!', 0, NULL, NO_DATE, THEME_ROOMINFO, "", "");
+    win_appendln(window, THEME_ROOMINFO, "");
 }
 
 void
@@ -324,7 +323,7 @@ mucwin_occupant_nick_change(ProfMucWin *mucwin, const char *const old_nick, cons
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_THEM, "", "** %s is now known as %s", old_nick, nick);
+    win_println(window, THEME_THEM, '!', "** %s is now known as %s", old_nick, nick);
 }
 
 void
@@ -333,7 +332,7 @@ mucwin_nick_change(ProfMucWin *mucwin, const char *const nick)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ME, "", "** You are now known as %s", nick);
+    win_println(window, THEME_ME, '!', "** You are now known as %s", nick);
 }
 
 void
@@ -355,7 +354,7 @@ mucwin_history(ProfMucWin *mucwin, const char *const nick, GDateTime *timestamp,
         g_string_append(line, message);
     }
 
-    win_print(window, '-', 0, timestamp, NO_COLOUR_DATE, 0, "", line->str);
+    win_print_history(window, timestamp, "%s", line->str);
     g_string_free(line, TRUE);
 
     plugins_on_room_history_message(mucwin->roomjid, nick, message, timestamp);
@@ -371,10 +370,10 @@ _mucwin_print_mention(ProfWin *window, const char *const message, const char *co
         pos = GPOINTER_TO_INT(curr->data);
 
         char *before_str = g_strndup(message + last_pos, pos - last_pos);
-        win_print(window, '-', 0, NULL, NO_DATE | NO_ME | NO_EOL, THEME_ROOMMENTION, "", before_str);
+        win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
         g_free(before_str);
         char *nick_str = g_strndup(message + pos, strlen(nick));
-        win_print(window, '-', 0, NULL, NO_DATE | NO_ME | NO_EOL, THEME_ROOMMENTION_TERM, "", nick_str);
+        win_append_highlight(window, THEME_ROOMMENTION_TERM, "%s", nick_str);
         g_free(nick_str);
 
         last_pos = pos + strlen(nick);
@@ -382,9 +381,9 @@ _mucwin_print_mention(ProfWin *window, const char *const message, const char *co
         curr = g_slist_next(curr);
     }
     if (last_pos < strlen(message)) {
-        win_print(window, '-', 0, NULL, NO_DATE | NO_ME, THEME_ROOMMENTION, "", &message[last_pos]);
+        win_appendln_highlight(window, THEME_ROOMMENTION, "%s", &message[last_pos]);
     } else {
-        win_print(window, '-', 0, NULL, NO_DATE | NO_ME, THEME_ROOMMENTION, "", "");
+        win_appendln_highlight(window, THEME_ROOMMENTION, "");
     }
 }
 
@@ -442,7 +441,7 @@ _mucwin_print_triggers(ProfWin *window, const char *const message, GList *trigge
 
     // no triggers found
     if (first_trigger_pos == -1) {
-        win_print(window, '-', 0, NULL, NO_DATE | NO_ME, THEME_ROOMTRIGGER, "", message);
+        win_appendln_highlight(window, THEME_ROOMTRIGGER, "%s", message);
     } else {
         if (first_trigger_pos > 0) {
             char message_section[strlen(message) + 1];
@@ -452,7 +451,7 @@ _mucwin_print_triggers(ProfWin *window, const char *const message, GList *trigge
                 i++;
             }
             message_section[i] = '\0';
-            win_print(window, '-', 0, NULL, NO_DATE | NO_ME | NO_EOL, THEME_ROOMTRIGGER, "", message_section);
+            win_append_highlight(window, THEME_ROOMTRIGGER, "%s", message_section);
         }
         char trigger_section[first_trigger_len + 1];
         int i = 0;
@@ -463,10 +462,10 @@ _mucwin_print_triggers(ProfWin *window, const char *const message, GList *trigge
         trigger_section[i] = '\0';
 
         if (first_trigger_pos + first_trigger_len < strlen(message)) {
-            win_print(window, '-', 0, NULL, NO_DATE | NO_ME | NO_EOL, THEME_ROOMTRIGGER_TERM, "", trigger_section);
+            win_append_highlight(window, THEME_ROOMTRIGGER_TERM, "%s", trigger_section);
             _mucwin_print_triggers(window, &message[first_trigger_pos + first_trigger_len], triggers);
         } else {
-            win_print(window, '-', 0, NULL, NO_DATE | NO_ME, THEME_ROOMTRIGGER_TERM, "", trigger_section);
+            win_appendln_highlight(window, THEME_ROOMTRIGGER_TERM, "%s", trigger_section);
         }
     }
 }
@@ -486,16 +485,16 @@ mucwin_message(ProfMucWin *mucwin, const char *const nick, const char *const mes
 
     if (g_strcmp0(nick, mynick) != 0) {
         if (g_slist_length(mentions) > 0) {
-            win_print(window, ch, 0, NULL, NO_ME | NO_EOL, THEME_ROOMMENTION, nick, "");
+            win_print_them(window, THEME_ROOMMENTION, ch, nick);
             _mucwin_print_mention(window, message, mynick, mentions);
         } else if (triggers) {
-            win_print(window, ch, 0, NULL, NO_ME | NO_EOL, THEME_ROOMTRIGGER, nick, "");
+            win_print_them(window, THEME_ROOMTRIGGER, ch, nick);
             _mucwin_print_triggers(window, message, triggers);
         } else {
-            win_print(window, ch, 0, NULL, NO_ME, THEME_TEXT_THEM, nick, message);
+            win_println_them_message(window, ch, nick, "%s", message);
         }
     } else {
-        win_print(window, ch, 0, NULL, 0, THEME_TEXT_ME, nick, message);
+        win_println_me_message(window, ch, mynick, "%s", message);
     }
 }
 
@@ -511,20 +510,20 @@ mucwin_requires_config(ProfMucWin *mucwin)
         ui_index = 0;
     }
 
-    win_print(window, '-', 0, NULL, 0, 0, "", "");
-    win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Room locked, requires configuration.");
-    win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Use '/room accept' to accept the defaults");
-    win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Use '/room destroy' to cancel and destroy the room");
-    win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Use '/room config' to edit the room configuration");
-    win_print(window, '-', 0, NULL, 0, 0, "", "");
+    win_println(window, THEME_DEFAULT, '-', "");
+    win_println(window, THEME_ROOMINFO, '!', "Room locked, requires configuration.");
+    win_println(window, THEME_ROOMINFO, '!', "Use '/room accept' to accept the defaults");
+    win_println(window, THEME_ROOMINFO, '!', "Use '/room destroy' to cancel and destroy the room");
+    win_println(window, THEME_ROOMINFO, '!', "Use '/room config' to edit the room configuration");
+    win_println(window, THEME_DEFAULT, '-', "");
 
     // currently in groupchat window
     if (wins_is_current(window)) {
-        status_bar_active(num);
+        status_bar_active(num, WIN_MUC, mucwin->roomjid);
 
     // not currently on groupchat window
     } else {
-        status_bar_new(num);
+        status_bar_new(num, WIN_MUC, mucwin->roomjid);
     }
 }
 
@@ -534,32 +533,21 @@ mucwin_subject(ProfMucWin *mucwin, const char *const nick, const char *const sub
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    int num = wins_get_num(window);
-
     if (subject) {
         if (nick) {
-            win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "*%s has set the room subject: ", nick);
-            win_vprint(window, '!', 0, NULL, NO_DATE, 0, "", "%s", subject);
+            win_print(window, THEME_ROOMINFO, '!', "*%s has set the room subject: ", nick);
+            win_appendln(window, THEME_DEFAULT, "%s", subject);
         } else {
-            win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "Room subject: ");
-            win_vprint(window, '!', 0, NULL, NO_DATE, 0, "", "%s", subject);
+            win_print(window, THEME_ROOMINFO, '!', "Room subject: ");
+            win_appendln(window, THEME_DEFAULT, "%s", subject);
         }
     } else {
         if (nick) {
-            win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "*%s has cleared the room subject.", nick);
+            win_println(window, THEME_ROOMINFO, '!', "*%s has cleared the room subject.", nick);
         } else {
-            win_vprint(window, '!', 0, NULL, 0, THEME_ROOMINFO, "", "Room subject cleared");
+            win_println(window, THEME_ROOMINFO, '!', "Room subject cleared");
         }
     }
-
-    // currently in groupchat window
-    if (wins_is_current(window)) {
-        status_bar_active(num);
-
-    // not currently on groupchat window
-    } else {
-        status_bar_active(num);
-    }
 }
 
 void
@@ -568,7 +556,7 @@ mucwin_kick_error(ProfMucWin *mucwin, const char *const nick, const char *const
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ERROR, "", "Error kicking %s: %s", nick, error);
+    win_println(window, THEME_ERROR, '!', "Error kicking %s: %s", nick, error);
 }
 
 void
@@ -579,16 +567,16 @@ mucwin_broadcast(ProfMucWin *mucwin, const char *const message)
     ProfWin *window = (ProfWin*)mucwin;
     int num = wins_get_num(window);
 
-    win_vprint(window, '!', 0, NULL, NO_EOL, THEME_ROOMINFO, "", "Room message: ");
-    win_vprint(window, '!', 0, NULL, NO_DATE, 0, "", "%s", message);
+    win_print(window, THEME_ROOMINFO, '!', "Room message: ");
+    win_appendln(window, THEME_DEFAULT, "%s", message);
 
     // currently in groupchat window
     if (wins_is_current(window)) {
-        status_bar_active(num);
+        status_bar_active(num, WIN_MUC, mucwin->roomjid);
 
     // not currently on groupchat window
     } else {
-        status_bar_new(num);
+        status_bar_new(num, WIN_MUC, mucwin->roomjid);
     }
 }
 
@@ -599,7 +587,7 @@ mucwin_affiliation_list_error(ProfMucWin *mucwin, const char *const affiliation,
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ERROR, "", "Error retrieving %s list: %s", affiliation, error);
+    win_println(window, THEME_ERROR, '!', "Error retrieving %s list: %s", affiliation, error);
 }
 
 void
@@ -609,17 +597,17 @@ mucwin_handle_affiliation_list(ProfMucWin *mucwin, const char *const affiliation
 
     ProfWin *window = (ProfWin*)mucwin;
     if (jids) {
-        win_vprint(window, '!', 0, NULL, 0, 0, "", "Affiliation: %s", affiliation);
+        win_println(window, THEME_DEFAULT, '!', "Affiliation: %s", affiliation);
         GSList *curr_jid = jids;
         while (curr_jid) {
             const char *jid = curr_jid->data;
-            win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", jid);
+            win_println(window, THEME_DEFAULT, '!', "  %s", jid);
             curr_jid = g_slist_next(curr_jid);
         }
-        win_print(window, '!', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '!', "");
     } else {
-        win_vprint(window, '!', 0, NULL, 0, 0, "", "No users found with affiliation: %s", affiliation);
-        win_print(window, '!', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '!', "No users found with affiliation: %s", affiliation);
+        win_println(window, THEME_DEFAULT, '!', "");
     }
 }
 
@@ -634,34 +622,34 @@ mucwin_show_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation)
     if (!occupants) {
         switch (affiliation) {
             case MUC_AFFILIATION_OWNER:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No owners found.");
+                win_println(window, THEME_DEFAULT, '!', "No owners found.");
                 break;
             case MUC_AFFILIATION_ADMIN:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No admins found.");
+                win_println(window, THEME_DEFAULT, '!', "No admins found.");
                 break;
             case MUC_AFFILIATION_MEMBER:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No members found.");
+                win_println(window, THEME_DEFAULT, '!', "No members found.");
                 break;
             case MUC_AFFILIATION_OUTCAST:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No outcasts found.");
+                win_println(window, THEME_DEFAULT, '!', "No outcasts found.");
                 break;
             default:
                 break;
         }
-        win_print(window, '-', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '-', "");
     } else {
         switch (affiliation) {
             case MUC_AFFILIATION_OWNER:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Owners:");
+                win_println(window, THEME_DEFAULT, '!', "Owners:");
                 break;
             case MUC_AFFILIATION_ADMIN:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Admins:");
+                win_println(window, THEME_DEFAULT, '!', "Admins:");
                 break;
             case MUC_AFFILIATION_MEMBER:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Members:");
+                win_println(window, THEME_DEFAULT, '!', "Members:");
                 break;
             case MUC_AFFILIATION_OUTCAST:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Outcasts:");
+                win_println(window, THEME_DEFAULT, '!', "Outcasts:");
                 break;
             default:
                 break;
@@ -672,16 +660,16 @@ mucwin_show_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation)
             Occupant *occupant = curr_occupant->data;
             if (occupant->affiliation == affiliation) {
                 if (occupant->jid) {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s (%s)", occupant->nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", occupant->nick, occupant->jid);
                 } else {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", occupant->nick);
+                    win_println(window, THEME_DEFAULT, '!', "  %s", occupant->nick);
                 }
             }
 
             curr_occupant = g_slist_next(curr_occupant);
         }
 
-        win_print(window, '-', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '-', "");
     }
 }
 
@@ -691,7 +679,7 @@ mucwin_role_list_error(ProfMucWin *mucwin, const char *const role, const char *c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ERROR, "", "Error retrieving %s list: %s", role, error);
+    win_println(window, THEME_ERROR, '!', "Error retrieving %s list: %s", role, error);
 }
 
 void
@@ -701,26 +689,26 @@ mucwin_handle_role_list(ProfMucWin *mucwin, const char *const role, GSList *nick
 
     ProfWin *window = (ProfWin*)mucwin;
     if (nicks) {
-        win_vprint(window, '!', 0, NULL, 0, 0, "", "Role: %s", role);
+        win_println(window, THEME_DEFAULT, '!', "Role: %s", role);
         GSList *curr_nick = nicks;
         while (curr_nick) {
             const char *nick = curr_nick->data;
             Occupant *occupant = muc_roster_item(mucwin->roomjid, nick);
             if (occupant) {
                 if (occupant->jid) {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s (%s)", nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", nick, occupant->jid);
                 } else {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", nick);
+                    win_println(window, THEME_DEFAULT, '!', "  %s", nick);
                 }
             } else {
-                win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", nick);
+                win_println(window, THEME_DEFAULT, '!', "  %s", nick);
             }
             curr_nick = g_slist_next(curr_nick);
         }
-        win_print(window, '!', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '!', "");
     } else {
-        win_vprint(window, '!', 0, NULL, 0, 0, "", "No occupants found with role: %s", role);
-        win_print(window, '!', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '!', "No occupants found with role: %s", role);
+        win_println(window, THEME_DEFAULT, '!', "");
     }
 }
 
@@ -735,28 +723,28 @@ mucwin_show_role_list(ProfMucWin *mucwin, muc_role_t role)
     if (!occupants) {
         switch (role) {
             case MUC_ROLE_MODERATOR:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No moderators found.");
+                win_println(window, THEME_DEFAULT, '!', "No moderators found.");
                 break;
             case MUC_ROLE_PARTICIPANT:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No participants found.");
+                win_println(window, THEME_DEFAULT, '!', "No participants found.");
                 break;
             case MUC_ROLE_VISITOR:
-                win_print(window, '!', 0, NULL, 0, 0, "", "No visitors found.");
+                win_println(window, THEME_DEFAULT, '!', "No visitors found.");
                 break;
             default:
                 break;
         }
-        win_print(window, '-', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '-', "");
     } else {
         switch (role) {
             case MUC_ROLE_MODERATOR:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Moderators:");
+                win_println(window, THEME_DEFAULT, '!', "Moderators:");
                 break;
             case MUC_ROLE_PARTICIPANT:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Participants:");
+                win_println(window, THEME_DEFAULT, '!', "Participants:");
                 break;
             case MUC_ROLE_VISITOR:
-                win_print(window, '!', 0, NULL, 0, 0, "", "Visitors:");
+                win_println(window, THEME_DEFAULT, '!', "Visitors:");
                 break;
             default:
                 break;
@@ -767,16 +755,16 @@ mucwin_show_role_list(ProfMucWin *mucwin, muc_role_t role)
             Occupant *occupant = curr_occupant->data;
             if (occupant->role == role) {
                 if (occupant->jid) {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s (%s)", occupant->nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", occupant->nick, occupant->jid);
                 } else {
-                    win_vprint(window, '!', 0, NULL, 0, 0, "", "  %s", occupant->nick);
+                    win_println(window, THEME_DEFAULT, '!', "  %s", occupant->nick);
                 }
             }
 
             curr_occupant = g_slist_next(curr_occupant);
         }
 
-        win_print(window, '-', 0, NULL, 0, 0, "", "");
+        win_println(window, THEME_DEFAULT, '-', "");
     }
 }
 
@@ -787,7 +775,7 @@ mucwin_affiliation_set_error(ProfMucWin *mucwin, const char *const jid, const ch
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ERROR, "", "Error setting %s affiliation for %s: %s", affiliation, jid, error);
+    win_println(window, THEME_ERROR, '!', "Error setting %s affiliation for %s: %s", affiliation, jid, error);
 }
 
 void
@@ -797,7 +785,7 @@ mucwin_role_set_error(ProfMucWin *mucwin, const char *const nick, const char *co
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_vprint(window, '!', 0, NULL, 0, THEME_ERROR, "", "Error setting %s role for %s: %s", role, nick, error);
+    win_println(window, THEME_ERROR, '!', "Error setting %s role for %s: %s", role, nick, error);
 }
 
 void
@@ -809,10 +797,10 @@ mucwin_info(ProfMucWin *mucwin)
     char *affiliation = muc_affiliation_str(mucwin->roomjid);
 
     ProfWin *window = (ProfWin*) mucwin;
-    win_vprint(window, '!', 0, NULL, 0, 0, "", "Room: %s", mucwin->roomjid);
-    win_vprint(window, '!', 0, NULL, 0, 0, "", "Affiliation: %s", affiliation);
-    win_vprint(window, '!', 0, NULL, 0, 0, "", "Role: %s", role);
-    win_print(window, '-', 0, NULL, 0, 0, "", "");
+    win_println(window, THEME_DEFAULT, '!', "Room: %s", mucwin->roomjid);
+    win_println(window, THEME_DEFAULT, '!', "Affiliation: %s", affiliation);
+    win_println(window, THEME_DEFAULT, '!', "Role: %s", role);
+    win_println(window, THEME_DEFAULT, '-', "");
 }
 
 void
