@@ -1,7 +1,7 @@
 /*
  * window.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -59,10 +59,21 @@ void win_show_status_string(ProfWin *window, const char *const from,
     const char *const show, const char *const status,
     GDateTime *last_activity, const char *const pre,
     const char *const default_show);
-void win_print_incoming_message(ProfWin *window, GDateTime *timestamp,
+
+void win_print_them(ProfWin *window, theme_item_t theme_item, char ch, const char *const them);
+void win_println_them_message(ProfWin *window, char ch, const char *const them, const char *const message, ...);
+void win_println_me_message(ProfWin *window, char ch, const char *const me, const char *const message, ...);
+
+void win_print_outgoing(ProfWin *window, const char ch, const char *const message, ...);
+void win_print_incoming(ProfWin *window, GDateTime *timestamp,
     const char *const from, const char *const message, prof_enc_t enc_mode);
-void win_print_with_receipt(ProfWin *window, const char show_char, int pad_indent, GTimeVal *tstamp, int flags,
-    theme_item_t theme_item, const char *const from, const char *const message, char *id);
+void win_print_history(ProfWin *window, GDateTime *timestamp, const char *const message, ...);
+
+void win_print_http_upload(ProfWin *window, const char *const message, char *url);
+
+void win_print_with_receipt(ProfWin *window, const char show_char, const char *const from, const char *const message,
+    char *id);
+
 void win_newline(ProfWin *window);
 void win_redraw(ProfWin *window);
 int win_roster_cols(void);
@@ -71,7 +82,6 @@ void win_sub_print(WINDOW *win, char *msg, gboolean newline, gboolean wrap, int
 void win_sub_newline_lazy(WINDOW *win);
 void win_mark_received(ProfWin *window, const char *const id);
 void win_update_entry_message(ProfWin *window, const char *const id, const char *const message);
-void win_update_entry_theme(ProfWin *window, const char *const id, theme_item_t theme_item);
 
 gboolean win_has_active_subwin(ProfWin *window);
 
