@@ -1,7 +1,7 @@
 /*
  * blocking.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -73,7 +73,7 @@ blocking_request(void)
     }
     blocked_ac = autocomplete_new();
 
-    char *id = create_unique_id("blocked_list_request");
+    char *id = connection_create_stanza_id("blocked_list_request");
     iq_id_handler_add(id, _blocklist_result_handler, NULL, NULL);
 
     xmpp_ctx_t *ctx = connection_get_ctx();
@@ -92,9 +92,9 @@ blocked_list(void)
 }
 
 char*
-blocked_ac_find(const char *const search_str)
+blocked_ac_find(const char *const search_str, gboolean previous)
 {
-    return autocomplete_complete(blocked_ac, search_str, TRUE);
+    return autocomplete_complete(blocked_ac, search_str, TRUE, previous);
 }
 
 void
@@ -115,7 +115,7 @@ blocked_add(char *jid)
 
     xmpp_ctx_t *ctx = connection_get_ctx();
 
-    char *id = create_unique_id("block");
+    char *id = connection_create_stanza_id("block");
     xmpp_stanza_t *iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
 
     xmpp_stanza_t *block = xmpp_stanza_new(ctx);
@@ -151,7 +151,7 @@ blocked_remove(char *jid)
 
     xmpp_ctx_t *ctx = connection_get_ctx();
 
-    char *id = create_unique_id("unblock");
+    char *id = connection_create_stanza_id("unblock");
     xmpp_stanza_t *iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
 
     xmpp_stanza_t *block = xmpp_stanza_new(ctx);
