@@ -1,7 +1,7 @@
 /*
  * connection.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -150,8 +150,13 @@ connection_connect(const char *const jid, const char *const passwd, const char *
 
     if (!tls_policy || (g_strcmp0(tls_policy, "force") == 0)) {
         xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_MANDATORY_TLS);
+    } else if (g_strcmp0(tls_policy, "trust") == 0) {
+        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_MANDATORY_TLS);
+        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_TRUST_TLS);
     } else if (g_strcmp0(tls_policy, "disable") == 0) {
         xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_DISABLE_TLS);
+    } else if (g_strcmp0(tls_policy, "legacy") == 0) {
+        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_LEGACY_SSL);
     }
 
 #ifdef HAVE_LIBMESODE
@@ -390,6 +395,27 @@ connection_free_uuid(char *uuid)
     }
 }
 
+char*
+connection_create_stanza_id(char *prefix)
+{
+    char *result = NULL;
+    GString *result_str = g_string_new("");
+    char *uuid = connection_create_uuid();
+
+    if (prefix) {
+        g_string_printf(result_str, "prof_%s_%s", prefix, uuid);
+    } else {
+        g_string_printf(result_str, "prof_%s", uuid);
+    }
+
+    connection_free_uuid(uuid);
+
+    result = result_str->str;
+    g_string_free(result_str, FALSE);
+
+    return result;
+}
+
 char*
 connection_get_domain(void)
 {
