@@ -1,7 +1,7 @@
 /*
  * muc.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -117,13 +117,18 @@ GSList* muc_occupants_by_affiliation(const char *const room, muc_affiliation_t a
 void muc_occupant_nick_change_start(const char *const room, const char *const new_nick, const char *const old_nick);
 char* muc_roster_nick_change_complete(const char *const room, const char *const nick);
 
+void muc_confserver_add(const char *const server);
+void muc_confserver_reset_ac(void);
+char* muc_confserver_find(const char *const search_str, gboolean previous);
+void muc_confserver_clear(void);
+
 void muc_invites_add(const char *const room, const char *const password);
 void muc_invites_remove(const char *const room);
 gint muc_invites_count(void);
-GSList* muc_invites(void);
+GList* muc_invites(void);
 gboolean muc_invites_contain(const char *const room);
 void muc_invites_reset_ac(void);
-char* muc_invites_find(const char *const search_str);
+char* muc_invites_find(const char *const search_str, gboolean previous);
 void muc_invites_clear(void);
 char* muc_invite_password(const char *const room);
 
@@ -133,7 +138,7 @@ char* muc_subject(const char *const room);
 void muc_pending_broadcasts_add(const char *const room, const char *const message);
 GList* muc_pending_broadcasts(const char *const room);
 
-char* muc_autocomplete(ProfWin *window, const char *const input);
+char* muc_autocomplete(ProfWin *window, const char *const input, gboolean previous);
 void muc_autocomplete_reset(const char *const room);
 
 gboolean muc_requires_config(const char *const room);
