@@ -1,7 +1,7 @@
 /*
  * roster_list.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -73,8 +73,6 @@ static gboolean _key_equals(void *key1, void *key2);
 static gboolean _datetimes_equal(GDateTime *dt1, GDateTime *dt2);
 static void _replace_name(const char *const current_name, const char *const new_name, const char *const barejid);
 static void _add_name_and_barejid(const char *const name, const char *const barejid);
-static gint _compare_name(PContact a, PContact b);
-static gint _compare_presence(PContact a, PContact b);
 
 void
 roster_create(void)
@@ -397,7 +395,7 @@ roster_get_contacts_by_presence(const char *const presence)
     while (g_hash_table_iter_next(&iter, &key, &value)) {
         PContact contact = (PContact)value;
         if (g_strcmp0(p_contact_presence(contact), presence) == 0) {
-            result = g_slist_insert_sorted(result, value, (GCompareFunc)_compare_name);
+            result = g_slist_insert_sorted(result, value, (GCompareFunc)roster_compare_name);
         }
     }
 
@@ -417,9 +415,9 @@ roster_get_contacts(roster_ord_t order)
 
     GCompareFunc cmp_func;
     if (order == ROSTER_ORD_PRESENCE) {
-        cmp_func = (GCompareFunc) _compare_presence;
+        cmp_func = (GCompareFunc) roster_compare_presence;
     } else {
-        cmp_func = (GCompareFunc) _compare_name;
+        cmp_func = (GCompareFunc) roster_compare_name;
     }
 
     g_hash_table_iter_init(&iter, roster->contacts);
@@ -444,7 +442,7 @@ roster_get_contacts_online(void)
     g_hash_table_iter_init(&iter, roster->contacts);
     while (g_hash_table_iter_next(&iter, &key, &value)) {
         if(strcmp(p_contact_presence(value), "offline"))
-            result = g_slist_insert_sorted(result, value, (GCompareFunc)_compare_name);
+            result = g_slist_insert_sorted(result, value, (GCompareFunc)roster_compare_name);
     }
 
     // return all contact structs
@@ -472,19 +470,19 @@ roster_has_pending_subscriptions(void)
 }
 
 char*
-roster_contact_autocomplete(const char *const search_str)
+roster_contact_autocomplete(const char *const search_str, gboolean previous)
 {
     assert(roster != NULL);
 
-    return autocomplete_complete(roster->name_ac, search_str, TRUE);
+    return autocomplete_complete(roster->name_ac, search_str, TRUE, previous);
 }
 
 char*
-roster_fulljid_autocomplete(const char *const search_str)
+roster_fulljid_autocomplete(const char *const search_str, gboolean previous)
 {
     assert(roster != NULL);
 
-    return autocomplete_complete(roster->fulljid_ac, search_str, TRUE);
+    return autocomplete_complete(roster->fulljid_ac, search_str, TRUE, previous);
 }
 
 GSList*
@@ -499,9 +497,9 @@ roster_get_group(const char *const group, roster_ord_t order)
 
     GCompareFunc cmp_func;
     if (order == ROSTER_ORD_PRESENCE) {
-        cmp_func = (GCompareFunc) _compare_presence;
+        cmp_func = (GCompareFunc) roster_compare_presence;
     } else {
-        cmp_func = (GCompareFunc) _compare_name;
+        cmp_func = (GCompareFunc) roster_compare_name;
     }
 
     g_hash_table_iter_init(&iter, roster->contacts);
@@ -526,7 +524,7 @@ roster_get_group(const char *const group, roster_ord_t order)
     return result;
 }
 
-GSList*
+GList*
 roster_get_groups(void)
 {
     assert(roster != NULL);
@@ -535,19 +533,19 @@ roster_get_groups(void)
 }
 
 char*
-roster_group_autocomplete(const char *const search_str)
+roster_group_autocomplete(const char *const search_str, gboolean previous)
 {
     assert(roster != NULL);
 
-    return autocomplete_complete(roster->groups_ac, search_str, TRUE);
+    return autocomplete_complete(roster->groups_ac, search_str, TRUE, previous);
 }
 
 char*
-roster_barejid_autocomplete(const char *const search_str)
+roster_barejid_autocomplete(const char *const search_str, gboolean previous)
 {
     assert(roster != NULL);
 
-    return autocomplete_complete(roster->barejid_ac, search_str, TRUE);
+    return autocomplete_complete(roster->barejid_ac, search_str, TRUE, previous);
 }
 
 static gboolean
@@ -605,8 +603,8 @@ _add_name_and_barejid(const char *const name, const char *const barejid)
     }
 }
 
-static gint
-_compare_name(PContact a, PContact b)
+gint
+roster_compare_name(PContact a, PContact b)
 {
     const char * utf8_str_a = NULL;
     const char * utf8_str_b = NULL;
@@ -645,8 +643,8 @@ _get_presence_weight(const char *presence)
     }
 }
 
-static gint
-_compare_presence(PContact a, PContact b)
+gint
+roster_compare_presence(PContact a, PContact b)
 {
     const char *presence_a = p_contact_presence(a);
     const char *presence_b = p_contact_presence(b);
@@ -663,6 +661,6 @@ _compare_presence(PContact a, PContact b)
 
     // otherwise order by name
     } else {
-        return _compare_name(a, b);
+        return roster_compare_name(a, b);
     }
 }
