@@ -1,7 +1,7 @@
 /*
  * roster_list.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -62,13 +62,15 @@ char* roster_barejid_from_name(const char *const name);
 GSList* roster_get_contacts(roster_ord_t order);
 GSList* roster_get_contacts_online(void);
 gboolean roster_has_pending_subscriptions(void);
-char* roster_contact_autocomplete(const char *const search_str);
-char* roster_fulljid_autocomplete(const char *const search_str);
+char* roster_contact_autocomplete(const char *const search_str, gboolean previous);
+char* roster_fulljid_autocomplete(const char *const search_str, gboolean previous);
 GSList* roster_get_group(const char *const group, roster_ord_t order);
-GSList* roster_get_groups(void);
-char* roster_group_autocomplete(const char *const search_str);
-char* roster_barejid_autocomplete(const char *const search_str);
+GList* roster_get_groups(void);
+char* roster_group_autocomplete(const char *const search_str, gboolean previous);
+char* roster_barejid_autocomplete(const char *const search_str, gboolean previous);
 GSList* roster_get_contacts_by_presence(const char *const presence);
 char* roster_get_msg_display_name(const char *const barejid, const char *const resource);
+gint roster_compare_name(PContact a, PContact b);
+gint roster_compare_presence(PContact a, PContact b);
 
 #endif
