@@ -158,7 +158,7 @@ void find_first_exists(void **state)
 
     char *search = strdup("B");
 
-    char *result = roster_contact_autocomplete(search);
+    char *result = roster_contact_autocomplete(search, FALSE);
     assert_string_equal("Bob", result);
     free(result);
     free(search);
@@ -172,7 +172,7 @@ void find_second_exists(void **state)
     roster_add("Dave", NULL, NULL, NULL, FALSE);
     roster_add("Bob", NULL, NULL, NULL, FALSE);
 
-    char *result = roster_contact_autocomplete("Dav");
+    char *result = roster_contact_autocomplete("Dav", FALSE);
     assert_string_equal("Dave", result);
     free(result);
     roster_destroy();
@@ -185,7 +185,7 @@ void find_third_exists(void **state)
     roster_add("Dave", NULL, NULL, NULL, FALSE);
     roster_add("Bob", NULL, NULL, NULL, FALSE);
 
-    char *result = roster_contact_autocomplete("Ja");
+    char *result = roster_contact_autocomplete("Ja", FALSE);
     assert_string_equal("James", result);
     free(result);
     roster_destroy();
@@ -198,7 +198,7 @@ void find_returns_null(void **state)
     roster_add("Dave", NULL, NULL, NULL, FALSE);
     roster_add("Bob", NULL, NULL, NULL, FALSE);
 
-    char *result = roster_contact_autocomplete("Mike");
+    char *result = roster_contact_autocomplete("Mike", FALSE);
     assert_null(result);
     roster_destroy();
 }
@@ -206,7 +206,7 @@ void find_returns_null(void **state)
 void find_on_empty_returns_null(void **state)
 {
     roster_create();
-    char *result = roster_contact_autocomplete("James");
+    char *result = roster_contact_autocomplete("James", FALSE);
     assert_null(result);
     roster_destroy();
 }
@@ -218,8 +218,8 @@ void find_twice_returns_second_when_two_match(void **state)
     roster_add("Jamie", NULL, NULL, NULL, FALSE);
     roster_add("Bob", NULL, NULL, NULL, FALSE);
 
-    char *result1 = roster_contact_autocomplete("Jam");
-    char *result2 = roster_contact_autocomplete(result1);
+    char *result1 = roster_contact_autocomplete("Jam", FALSE);
+    char *result2 = roster_contact_autocomplete(result1, FALSE);
     assert_string_equal("Jamie", result2);
     free(result1);
     free(result2);
@@ -240,11 +240,11 @@ void find_five_times_finds_fifth(void **state)
     roster_add("Jamy", NULL, NULL, NULL, FALSE);
     roster_add("Jamz", NULL, NULL, NULL, FALSE);
 
-    char *result1 = roster_contact_autocomplete("Jam");
-    char *result2 = roster_contact_autocomplete(result1);
-    char *result3 = roster_contact_autocomplete(result2);
-    char *result4 = roster_contact_autocomplete(result3);
-    char *result5 = roster_contact_autocomplete(result4);
+    char *result1 = roster_contact_autocomplete("Jam", FALSE);
+    char *result2 = roster_contact_autocomplete(result1, FALSE);
+    char *result3 = roster_contact_autocomplete(result2, FALSE);
+    char *result4 = roster_contact_autocomplete(result3, FALSE);
+    char *result5 = roster_contact_autocomplete(result4, FALSE);
     assert_string_equal("Jamo", result5);
     free(result1);
     free(result2);
@@ -261,9 +261,9 @@ void find_twice_returns_first_when_two_match_and_reset(void **state)
     roster_add("Jamie", NULL, NULL, NULL, FALSE);
     roster_add("Bob", NULL, NULL, NULL, FALSE);
 
-    char *result1 = roster_contact_autocomplete("Jam");
+    char *result1 = roster_contact_autocomplete("Jam", FALSE);
     roster_reset_search_attempts();
-    char *result2 = roster_contact_autocomplete(result1);
+    char *result2 = roster_contact_autocomplete(result1, FALSE);
     assert_string_equal("James", result2);
     free(result1);
     free(result2);
@@ -275,10 +275,10 @@ void add_contact_with_no_group(void **state)
     roster_create();
     roster_add("person@server.org", NULL, NULL, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 0);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 0);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -290,14 +290,14 @@ void add_contact_with_group(void **state)
     groups = g_slist_append(groups, strdup("friends"));
     roster_add("person@server.org", NULL, groups, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 1);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 1);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "friends");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -310,17 +310,17 @@ void add_contact_with_two_groups(void **state)
     groups = g_slist_append(groups, strdup("work"));
     roster_add("person@server.org", NULL, groups, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 2);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 2);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "friends");
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "work");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -334,20 +334,20 @@ void add_contact_with_three_groups(void **state)
     groups = g_slist_append(groups, strdup("stuff"));
     roster_add("person@server.org", NULL, groups, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 3);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 3);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "friends");
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "work");
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "stuff");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -369,26 +369,26 @@ void add_contact_with_three_groups_update_adding_two(void **state)
     groups2 = g_slist_append(groups2, strdup("people"));
     roster_update("person@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 5);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 5);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "friends");
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "work");
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "stuff");
-    found = g_slist_find_custom(groups_res, "things", g_strcmp0);
+    found = g_list_find_custom(groups_res, "things", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "things");
-    found = g_slist_find_custom(groups_res, "people", g_strcmp0);
+    found = g_list_find_custom(groups_res, "people", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "people");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -407,17 +407,17 @@ void add_contact_with_three_groups_update_removing_one(void **state)
     groups2 = g_slist_append(groups2, strdup("stuff"));
     roster_update("person@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 2);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 2);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "friends");
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "stuff");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -435,14 +435,14 @@ void add_contact_with_three_groups_update_removing_two(void **state)
     groups2 = g_slist_append(groups2, strdup("stuff"));
     roster_update("person@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 1);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 1);
 
-    GSList *found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
     assert_string_equal(found->data, "stuff");
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -458,10 +458,10 @@ void add_contact_with_three_groups_update_removing_three(void **state)
 
     roster_update("person@server.org", NULL, NULL, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 0);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 0);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -480,15 +480,15 @@ void add_contact_with_three_groups_update_two_new(void **state)
     groups2 = g_slist_append(groups2, strdup("somepeople"));
     roster_update("person@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 2);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 2);
 
-    GSList *found = g_slist_find_custom(groups_res, "newfriends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "newfriends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "somepeople", g_strcmp0);
+    found = g_list_find_custom(groups_res, "somepeople", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -504,10 +504,10 @@ void add_remove_contact_groups(void **state)
 
     roster_remove("person@server.org", "person@server.org");
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 0);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 0);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -526,21 +526,21 @@ void add_contacts_with_different_groups(void **state)
     groups2 = g_slist_append(groups2, strdup("somepeople"));
     roster_add("bob@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 5);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 5);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "newfriends", g_strcmp0);
+    found = g_list_find_custom(groups_res, "newfriends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "somepeople", g_strcmp0);
+    found = g_list_find_custom(groups_res, "somepeople", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -560,17 +560,17 @@ void add_contacts_with_same_groups(void **state)
     groups2 = g_slist_append(groups2, strdup("stuff"));
     roster_add("bob@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 3);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 3);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -590,19 +590,19 @@ void add_contacts_with_overlapping_groups(void **state)
     groups2 = g_slist_append(groups2, strdup("different"));
     roster_add("bob@server.org", NULL, groups2, NULL, FALSE);
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 4);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 4);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "different", g_strcmp0);
+    found = g_list_find_custom(groups_res, "different", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
 
@@ -624,16 +624,16 @@ void remove_contact_with_remaining_in_group(void **state)
 
     roster_remove("bob@server.org", "bob@server.org");
 
-    GSList *groups_res = roster_get_groups();
-    assert_int_equal(g_slist_length(groups_res), 3);
+    GList *groups_res = roster_get_groups();
+    assert_int_equal(g_list_length(groups_res), 3);
 
-    GSList *found = g_slist_find_custom(groups_res, "friends", g_strcmp0);
+    GList *found = g_list_find_custom(groups_res, "friends", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "work", g_strcmp0);
+    found = g_list_find_custom(groups_res, "work", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
-    found = g_slist_find_custom(groups_res, "stuff", g_strcmp0);
+    found = g_list_find_custom(groups_res, "stuff", (GCompareFunc)g_strcmp0);
     assert_true(found != NULL);
 
-    g_slist_free_full(groups_res, g_free);
+    g_list_free_full(groups_res, free);
     roster_destroy();
 }
