@@ -43,17 +43,9 @@ expect_any_cons_show_error(void)
 }
 
 void
-expect_ui_current_print_line(char *message)
+expect_win_println(char *message)
 {
-    expect_string(ui_current_print_line, output, message);
-}
-
-void
-expect_ui_current_print_formatted_line(char show_char, int attrs, char *message)
-{
-    expect_value(ui_current_print_formatted_line, show_char, show_char);
-    expect_value(ui_current_print_formatted_line, attrs, attrs);
-    expect_string(ui_current_print_formatted_line, output, message);
+    expect_string(win_println, output, message);
 }
 
 // stubs
@@ -132,9 +124,6 @@ void ui_current_print_formatted_line(const char show_char, int attrs, const char
     va_end(args);
 }
 
-void ui_current_error_line(const char * const msg) {}
-void ui_win_error_line(ProfWin *window, const char * const msg) {}
-
 void ui_close_win(int index) {}
 
 int ui_win_unread(int index)
@@ -261,7 +250,7 @@ void ui_handle_error(const char * const err_msg)
 
 void ui_clear_win_title(void) {}
 void ui_goodbye_title(void) {}
-void mucconfwin_handle_configuration(ProfMucConfWin *confwin, DataForm *form) {}
+void confwin_handle_configuration(ProfConfWin *confwin, DataForm *form) {}
 void ui_handle_room_configuration_form_error(const char * const roomjid, const char * const message) {}
 void ui_handle_room_config_submit_result(const char * const roomjid) {}
 void ui_handle_room_config_submit_result_error(const char * const roomjid, const char * const message) {}
@@ -274,10 +263,10 @@ void mucwin_role_set_error(ProfMucWin *mucwin, const char * const nick, const ch
 void mucwin_role_list_error(ProfMucWin *mucwin, const char * const role, const char * const error) {}
 void mucwin_handle_role_list(ProfMucWin *mucwin, const char * const role, GSList *nicks) {}
 void mucwin_kick_error(ProfMucWin *mucwin, const char * const nick, const char * const error) {}
-void mucconfwin_show_form(ProfMucConfWin *confwin) {}
-void mucconfwin_show_form_field(ProfMucConfWin *confwin, DataForm *form, char *tag) {}
-void mucconfwin_form_help(ProfMucConfWin *confwin) {}
-void mucconfwin_field_help(ProfMucConfWin *confwin, char *tag) {}
+void confwin_show_form(ProfConfWin *confwin) {}
+void confwin_show_form_field(ProfConfWin *confwin, DataForm *form, char *tag) {}
+void confwin_form_help(ProfConfWin *confwin) {}
+void confwin_field_help(ProfConfWin *confwin, char *tag) {}
 void ui_show_lines(ProfWin *window, gchar** lines) {}
 void ui_redraw_all_room_rosters(void) {}
 void ui_show_all_room_rosters(void) {}
@@ -419,7 +408,7 @@ void cons_check_version(gboolean not_available_msg) {}
 void cons_show_typing(const char * const barejid) {}
 void cons_show_incoming_room_message(const char *const nick, const char *const room, const int win_index, gboolean mention, GList *triggers, int unread) {}
 void cons_show_incoming_message(const char * const short_from, const int win_index, int unread) {}
-void cons_show_room_invites(GSList *invites) {}
+void cons_show_room_invites(GList *invites) {}
 void cons_show_received_subs(void) {}
 void cons_show_sent_subs(void) {}
 void cons_alert(void) {}
@@ -440,7 +429,7 @@ void cons_encwarn_setting(void) {}
 void cons_time_setting(void) {}
 void cons_mouse_setting(void) {}
 void cons_statuses_setting(void) {}
-void cons_titlebar_setting(void) {}
+void cons_wintitle_setting(void) {}
 void cons_notify_setting(void) {}
 void cons_states_setting(void) {}
 void cons_outtype_setting(void) {}
@@ -456,7 +445,10 @@ void cons_autoaway_setting(void) {}
 void cons_reconnect_setting(void) {}
 void cons_autoping_setting(void) {}
 void cons_autoconnect_setting(void) {}
+void cons_rooms_cache_setting(void) {}
 void cons_inpblock_setting(void) {}
+void cons_winpos_setting(void) {}
+void cons_statusbar_setting(void) {}
 void cons_tray_setting(void) {}
 
 void cons_show_contact_online(PContact contact, Resource *resource, GDateTime *last_activity)
@@ -475,8 +467,8 @@ void title_bar_set_presence(contact_presence_t presence) {}
 
 // status bar
 void status_bar_inactive(const int win) {}
-void status_bar_active(const int win) {}
-void status_bar_new(const int win) {}
+void status_bar_active(const int win, win_type_t type, char *identifier) {}
+void status_bar_new(const int win, win_type_t type, char *identifier) {}
 void status_bar_set_all_inactive(void) {}
 
 // roster window
@@ -502,7 +494,7 @@ ProfWin* win_create_muc(const char * const roomjid)
 {
     return NULL;
 }
-ProfWin* win_create_muc_config(const char * const title, DataForm *form)
+ProfWin* win_create_config(const char *const title, DataForm *form, ProfConfWinCallback submit, ProfConfWinCallback cancel, const void *userdata)
 {
     return NULL;
 }
@@ -515,6 +507,11 @@ ProfWin* win_create_plugin(const char *const plugin_name, const char * const tag
     return NULL;
 }
 
+char* win_get_tab_identifier(ProfWin *window)
+{
+    return NULL;
+}
+
 void win_update_virtual(ProfWin *window) {}
 void win_free(ProfWin *window) {}
 gboolean win_notify_remind(ProfWin *window)
@@ -531,8 +528,19 @@ void win_hide_subwin(ProfWin *window) {}
 void win_show_subwin(ProfWin *window) {}
 void win_refresh_without_subwin(ProfWin *window) {}
 void win_refresh_with_subwin(ProfWin *window) {}
-void win_print(ProfWin *window, const char show_char, int pad_indent, GDateTime *timestamp, int flags, theme_item_t theme_item, const char * const from, const char * const message) {}
-void win_vprint(ProfWin *window, const char show_char, int pad_indent, GDateTime *timestamp, int flags, theme_item_t theme_item, const char * const from, const char * const message, ...) {}
+
+void win_println(ProfWin *window, theme_item_t theme, const char ch, const char *const message, ...)
+{
+    va_list args;
+    va_start(args, message);
+    vsnprintf(output, sizeof(output), message, args);
+    check_expected(output);
+    va_end(args);
+}
+
+void win_print(ProfWin *window, theme_item_t theme_item, const char ch, const char *const message, ...) {}
+void win_appendln(ProfWin *window, theme_item_t theme_item, const char *const message, ...) {}
+
 char* win_get_title(ProfWin *window)
 {
     return NULL;
@@ -541,10 +549,9 @@ void win_show_occupant(ProfWin *window, Occupant *occupant) {}
 void win_show_occupant_info(ProfWin *window, const char * const room, Occupant *occupant) {}
 void win_show_contact(ProfWin *window, PContact contact) {}
 void win_show_info(ProfWin *window, PContact contact) {}
-void win_println(ProfWin *window, int pad, const char * const message) {}
-void win_vprintln_ch(ProfWin *window, char ch, const char *const message, ...) {}
+void win_println_indent(ProfWin *window, int pad, const char * const message, ...) {}
 void win_clear(ProfWin *window) {}
-char* win_get_string(ProfWin *window)
+char* win_to_string(ProfWin *window)
 {
     return NULL;
 }
